@@ -32,9 +32,14 @@ G_BEGIN_DECLS
 
 struct stat;
 
+#ifndef GNOME_VFS_DISABLE_DEPRECATED
 void 	     gnome_vfs_mime_shutdown 				(void);
+#endif
 
+#ifndef GNOME_VFS_DISABLE_DEPRECATED
 const char  *gnome_vfs_mime_type_from_name			(const char        *filename);
+#endif
+
 const char  *gnome_vfs_mime_type_from_name_or_default	        (const char        *filename,
 								 const char        *defaultv);
 
@@ -47,12 +52,16 @@ const char  *gnome_vfs_get_mime_type_from_uri			(GnomeVFSURI       *uri);
 const char  *gnome_vfs_get_mime_type_from_file_data		(GnomeVFSURI       *uri);
 #endif
 
-const char  *gnome_vfs_get_file_mime_type 			(const char        *path, 
-								 const struct stat *optional_stat_info,
-								 gboolean           suffix_only);
+const char * gnome_vfs_get_file_mime_type_fast      (const char        *path,
+						     const struct stat *optional_stat_info);
+const char  *gnome_vfs_get_file_mime_type           (const char        *path,
+						     const struct stat *optional_stat_info,
+						     gboolean           suffix_only);
+gboolean     gnome_vfs_mime_type_is_supertype       (const char        *mime_type);
+char	    *gnome_vfs_get_supertype_from_mime_type (const char        *mime_type);
 
-gboolean    gnome_vfs_mime_type_is_supertype			(const char        *mime_type);
-char	    *gnome_vfs_get_supertype_from_mime_type		(const char        *mime_type);							 
+void         gnome_vfs_mime_info_cache_reload (const char *dir);
+void         gnome_vfs_mime_reload            (void);
 
 G_END_DECLS
 
