@@ -291,7 +291,7 @@ typedef struct {
 	   
 } MethodSchemes;
 
-static MethodSchemes supported_schemes[] = {
+static const MethodSchemes supported_schemes[] = {
 
 	{"http",  FALSE, "http",  FALSE},
 	{"dav",   FALSE, "http",  TRUE},
@@ -303,7 +303,7 @@ static MethodSchemes supported_schemes[] = {
 static char *
 resolve_alias (const char *scheme) 
 {
-	MethodSchemes *iter;
+	const MethodSchemes *iter;
 	
 	if (scheme == NULL)
 		return NULL;
@@ -400,6 +400,7 @@ G_LOCK_DEFINE (auth_cache);
 static GHashTable *auth_cache_basic;
 static GHashTable *auth_cache_proxy;
 static guint       cleanup_id;
+static guint       module_refcount = 0;
 
 typedef struct {
 	
@@ -988,11 +989,11 @@ typedef enum {
 	
 } HttpMethods;
 
-struct HttpMethod {
+static struct HttpMethod {
 	char           *string;
 	HttpMethods  type;
 	
-} http_all_methods [] = {
+} const http_all_methods [] = {
 	{"OPTIONS", ALLOW_OPTIONS},
 	{"GET", ALLOW_GET},
 	{"HEAD", ALLOW_HEAD},
@@ -1015,12 +1016,12 @@ static GHashTable *http_methods = NULL;
 static void
 quick_allow_lookup_init (void)
 {
-	struct HttpMethod *iter;
+	const struct HttpMethod *iter;
 	
 	http_methods = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	for (iter = http_all_methods; iter->string; iter++) {
-		g_hash_table_insert (http_methods, iter->string, iter);
+		g_hash_table_insert (http_methods, iter->string, (struct HttpMethod *)iter);
 	}
 }
 
@@ -1190,90 +1191,98 @@ propfind_result (void *userdata, const char *href, const ne_prop_result_set *set
 	char *unesc_path;
 
 	ctx = (PropfindContext *) userdata;
-	
+
 	if (ne_uri_parse (href, &uri))
 		return;
-    
+
 	if (uri.path == NULL) {
 		ne_uri_free (&uri);
 		return;
 	}
-	
+
 	DEBUG_HTTP_2 ("href: %s", href);
-	
+
 	info = gnome_vfs_file_info_new ();
 	unesc_path = ne_path_unescape (uri.path);
 	info->name = g_path_get_basename (unesc_path);
 	NE_FREE (unesc_path);
-	
+
 	DEBUG_HTTP_2 ("Comparing: \n\t[%s] \n\t[%s]", ctx->path, uri.path);
 	if (ne_path_compare (ctx->path, uri.path) == 0) {
 		DEBUG_HTTP_3 ("target");
 		ctx->target = info;
-		
+
 		/* Set the etag on target */
 		/* We are not requesting the etag information at 
 		 * the moment so don't even check for it 
 		value = ne_propset_value (set, &file_info_props[5]);
-		
+
 		if (value != NULL)
 			ctx->etag = g_strdup (value);
-	
-		*/	
+
+		*/
 	} else {
 		ctx->children = g_list_append (ctx->children, info);
 	}
-	
+
 	ne_uri_free (&uri);
-	
+
 	/* getlastmodified */
 	value = ne_propset_value (set, &file_info_props[0]);
-	
+
 	if (value && gnome_vfs_atotm (value, &time)) {
 		info->mtime = time;
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MTIME;
 	}
-	
+
 	/* creationdate */
 	value = ne_propset_value (set, &file_info_props[1]);
-	
+
 	if (value && gnome_vfs_atotm (value, &time)) {
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_CTIME;
 	}
-	
+
 	/* resourctype */
 	value = ne_propset_value (set, &file_info_props[2]);
-	
+
 	info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
-	
+
 	if (value != NULL && strstr (value, "collection")) {
 		info->mime_type = g_strdup (HTTP_DIR_MIME_TYPE);
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 		info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
-		
+
 		return;
 	} else {
 		info->type = GNOME_VFS_FILE_TYPE_REGULAR;
 	}
-	
-	/* The following properties only apply to files */
-	
+
+	/*** The following properties only apply to files ***/
+
 	/* getcontenttype */
-	value = ne_propset_value (set, &file_info_props[3]);
-		
+
+	/* We only evaluate the getcontenttype filed if it hasn't already
+	 * been set (i.e. to HTTP_DIR_MIME_TYPE) so we can indicate the
+	 * directory mime-type */
+	if (! (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE)) {
+		value = ne_propset_value (set, &file_info_props[3]);
+	} else {
+		value = NULL;
+	}
+
 	if (! value) {
 		value = gnome_vfs_mime_type_from_name (info->name);
 	}
-	
+
 	if (value) {
 		info->mime_type = g_strdup (value);
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 	}
-	
+
 	/* getcontentlength */
-	
+
 	value = ne_propset_value (set, &file_info_props[4]);
-	
+
 	if (value && header_value_to_number (value, &size)) {
 		info->size = size;
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
@@ -1608,6 +1617,8 @@ http_acquire_connection (HttpContext *context)
 		return GNOME_VFS_ERROR_INTERNAL;
 	}
 
+	ne_set_read_timeout (session, 30);
+
 	user_agent = getenv (CUSTOM_USER_AGENT_VARIABLE);
 	   
 	if (user_agent == NULL) {
@@ -1710,7 +1721,8 @@ http_context_set_uri (HttpContext *context, GnomeVFSURI *uri)
 					      | GNOME_VFS_URI_HIDE_PASSWORD
 					      | GNOME_VFS_URI_HIDE_HOST_NAME
 					      | GNOME_VFS_URI_HIDE_HOST_PORT
-					      | GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+					      | GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD
+					      | GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER);
 	if (uri_string[0] == '\0') {
 		g_free (uri_string);
 		uri_string = g_strdup ("/");
@@ -1838,47 +1850,68 @@ http_get_file_info (HttpContext *context, GnomeVFSFileInfo *info)
 		
 		result = http_follow_redirect (context);
 		
-		if (result == GNOME_VFS_OK)
+		if (result == GNOME_VFS_OK) {
 			goto propfind_start;
+		} else {
+			return result;
+		}
 	}
-	
+
 	req = ne_propfind_get_request (pfh);
 	result = resolve_result (res, req);
 	DEBUG_HTTP ("%d, %d", res, ne_get_status (req)->code);
 	ne_propfind_destroy (pfh);
-	
-	/* Let's be very cautious here! If the server doesn't respond with a 
-	   207 here just fall back to HEAD because some server server (eg. gws) 
+
+	/* Let's be very cautious here! If the server doesn't respond with a
+	   207 here just fall back to HEAD because some server server (eg. gws)
 	   close the connection on an unknown command, most (stupid) php scripts
 	   will treat PROPFIND as GET and some servers may deny us PROFIND but
-	   allow us HEAD. */
-	if (res == NE_OK && ne_get_status (req)->code == 207) {
+	   allow us HEAD. 
+	   The only exception is if we receive a 404 */
 
-		if (pfctx.target != NULL) {
-			gnome_vfs_file_info_copy (info, pfctx.target);	
-		} else {
+	if (res == NE_OK) {
+		gboolean have_result = FALSE;
+		const ne_status *status = ne_get_status (req);
+
+		if (status->code == 207) {
+
+			have_result = TRUE;
+
+			if (pfctx.target != NULL) {
+				gnome_vfs_file_info_copy (info, pfctx.target);
+			} else {
+				result = GNOME_VFS_ERROR_NOT_FOUND;
+			}
+
+		} else if (status->code == 404) {
+			have_result = TRUE;
 			result = GNOME_VFS_ERROR_NOT_FOUND;
 		}
-		
-		propfind_context_clear (&pfctx);
-		return result;
+
+		if (have_result == TRUE) {
+			propfind_context_clear (&pfctx);
+			return result;
+		}
 	}
-	
+
 	propfind_context_clear (&pfctx);
 	DEBUG_HTTP ("!! Fallthrough to head (%s)", gnome_vfs_result_to_string (result));
-	
- head_start:	
+
+ head_start:
 	req  = ne_request_create (context->session, "HEAD", context->path);
 
 	res = ne_request_dispatch (req);
 
 	if (res == NE_REDIRECT) {
-		result = http_follow_redirect (context);
-		
+		ne_request_destroy (req);
+		req = NULL;
+
+		result = http_follow_redirect (context);		
+	
 		if (result == GNOME_VFS_OK) {
-			ne_request_destroy (req);
-			req = NULL;
 			goto head_start;
+		} else {
+			return result;
 		}
 	}
 
@@ -1983,7 +2016,12 @@ http_options (HttpContext *hctx)
 	if (res == NE_REDIRECT) {
 		ne_request_destroy (req);
 		result = http_follow_redirect (hctx);
-		goto options_start;
+		
+		if (result == GNOME_VFS_OK) {
+			goto options_start;
+		} else {
+			return result;
+		}
 	}
 
 	result = resolve_result (res, req);
@@ -2266,7 +2304,7 @@ get_retry:
 		
 		if (res == NE_REDIRECT) {
 			result = http_follow_redirect (hctx);
-				
+	
 			if (result == GNOME_VFS_OK)
 				goto get_start;
 		}
@@ -2415,6 +2453,7 @@ do_create (GnomeVFSMethod	 *method,
 
 	hctx = handle->context;	
 
+put_start:
 	req = ne_request_create (hctx->session, "PUT", hctx->path);
 
 	/* Apache seems to not handle "If-None-Match: *" correctly 
@@ -2437,6 +2476,20 @@ do_create (GnomeVFSMethod	 *method,
 	ne_set_request_body_buffer (req, NULL, 0);
 	
 	res = ne_request_dispatch (req);
+
+	if (res == NE_REDIRECT) {
+		ne_request_destroy (req);
+		req = NULL;
+		
+		result = http_follow_redirect (hctx);
+		
+		if (result == GNOME_VFS_OK) {
+			goto put_start;
+		} else {
+			return result;
+		}
+	}
+
 	result 	= resolve_result (res, req);
 	ne_request_destroy (req);
 	
@@ -3296,20 +3349,24 @@ static GnomeVFSMethod http_method = {
 GnomeVFSMethod *
 vfs_module_init (const char *method_name, const char *args)
 {
-	proxy_init ();
-	/* ne_debug_init (stdout, 0xfffe); */
-	neon_session_pool_init ();
-	http_auth_cache_init ();
-	quick_allow_lookup_init ();
-	
+	if (module_refcount++ == 0) {
+		proxy_init ();
+		/* ne_debug_init (stdout, 0xfffe); */
+		neon_session_pool_init ();
+		http_auth_cache_init ();
+		quick_allow_lookup_init ();
+	}
+
 	return &http_method;
 }
 
 void
 vfs_module_shutdown (GnomeVFSMethod *method)
 {
-	quit_allow_lookup_destroy ();
-	http_auth_cache_shutdown ();
-	neon_session_pool_shutdown ();
-	proxy_shutdown ();
+	if (--module_refcount == 0) {
+		quit_allow_lookup_destroy ();
+		http_auth_cache_shutdown ();
+		neon_session_pool_shutdown ();
+		proxy_shutdown ();
+	}
 }
